#!/usr/bin/ruby
#
# = Synopsys
#
# This module is the main entry point of the application. Within it the ESXServer
# class is defined and other relevant modules and methods are included.
#
# = Usage
#
# See example scripts for information on useage.
#
# = Author
#
# John Fitzpatrick, MWR Infosecurity, UK.
# <john.fitzpatrick[-at-]mwrinfosecurity.com>, <john[-at-]tinternet.org.uk>
#
# = License
#
# This file may be used under the terms of the GNU General Public
# License version 3.0 as published by the Free Software Foundation
# and appearing in the file LICENSE.txt included in the packaging of
# this file.
#
# = Copyright
#
# Copyright (c) 2008 John Fitzpatrick <john[-at-]tinternet.org.uk>
#

module VMware


  # this module will provide the methods for various operations against the 
  # VI sdk, such as retrieveing service content and specific values
  module Vi

    # include the required code for the VI sdk web service
    require "#{File.dirname(__FILE__)}/Vim25ClassDriver.rb"
    include Vim25

    # include different ESX VI SDK modules
    require "#{File.dirname(__FILE__)}/vim25_Auth.rb"
    include Auth

    require "#{File.dirname(__FILE__)}/vim25_Firewall.rb"
    include Firewall

    require "#{File.dirname(__FILE__)}/vim25_Network.rb"
    include Network

    require "#{File.dirname(__FILE__)}/vim25_Patching.rb"
    include Patching

    require "#{File.dirname(__FILE__)}/vim25_Version.rb"
    include Version

    require "#{File.dirname(__FILE__)}/vim25_VirtualMachine.rb"
    include VirtualMachine

    #
    #
    # method to wrap up a retrieveProperties request
    def retrieveProperty(object, skip, type, pathSet, all)
       # an object spec specifies the managed object which the filter begins at 
      objectSpec = ObjectSpec.new
      objectSpec.obj = object
      objectSpec.skip = skip # whether or not to report this managed object's properties. no=true

      # property spec specifies a managed object from whic to extract data
      propertySpec = PropertySpec.new
      propertySpec.type = type # the type of object where the search path starts
      propertySpec.pathSet = [pathSet] # search path
      propertySpec.all = all # set to true to ignore the pathSet property and return all properties

      # property filter spec defines the property data to include in a filter
      propFilterSpec = PropertyFilterSpec.new
      propFilterSpec.objectSet = [objectSpec]
      propFilterSpec.propSet = propertySpec

      requestType = RetrievePropertiesRequestType.new(@servicecontent.returnval.propertyCollector)
      requestType.specSet = propFilterSpec

      return @driver.retrieveProperties(requestType)

    end # end retrieveProperty method


    #
    #
    # later requests will require a reference to a hostSystem, this is
    # retrieved by this method through calling findByIP
    # - this method does not return a value, it sets a class variable which
    #   can be accessed later
    def setHostSystem
      # retrieve the host connection info for the ESX host based on its IP address
      srchindex = ManagedObjectReference.new(@servicecontent.returnval.searchIndex)
      srchindex.xmlattr_type = "SearchIndex"
      findbyipreq = FindByIpRequestType.new(srchindex)
      findbyipreq.ip = @ip
      findbyipreq.vmSearch = false
      @hostSystem = @driver.findByIp(findbyipreq)
    end # setHostSystem

    #
    #
    # this method obtains a reference to the conninfo which is required by 
    # many later requests
    def setConninfo
      host = ManagedObjectReference.new(@hostSystem.returnval)
      host.xmlattr_type = "HostSystem"
      qryhstconn = QueryHostConnectionInfoRequestType.new(host)
      @conninfo = @driver.queryHostConnectionInfo(qryhstconn)
    end # end setConninfo


    #
    #
    # a lot of later requests will require access to ConfigManager so
    # this method is made available in order to grab a reference to the
    # ConfigManager
    def setConfigManager
      @configManager = retrieveProperty(@hostSystem.returnval,
                                          true,
                                          "HostSystem",
                                          "configManager",
                                          true)
    end # end setConfigManager method

    #
    #
    # the root folder basically holds all data stored on the syste, including
    # the virtual machines and their configurations. so this is very useful to 
    # retrieve
    def setRootFolder
      @rootFolder = retrieveProperty(@servicecontent.returnval.rootFolder,
                                          true,
                                          "Folder",
                                          "childEntity",
                                          true)
    end # end setRootFolder method

    #
    #
    # the root folder basically holds all data stored on the system, including
    # the virtual machines and their configurations. so this is very useful to 
    # retrieve
    def setDatacenter
      # NOTE: ha-datacenter is held in an array so needs extracting first([0])
      @datacenter = retrieveProperty(@rootFolder[0].propSet[1].val[0],
                                          false,
                                          "Datacenter",
                                          "name",
                                          true) # return everything not just name
    end # end setDatacenter method

    #
    #
    # whilst a datacenter holds VMs they are actually held in a datastore within
    # the datacenter and so this needs to be retrieved
    def setDatastore
      # NOTE: really this (and setDatacenter() should search for this field rather
      # than (in this case) just going for the value 4 incase this changes in the
      # future
      @datastore = retrieveProperty(@datacenter[0].propSet[4].val[0],
                                          false,
                                          "Datastore",
                                          "vm",
                                          true) # return everything not just name
    end # end setDatastore

    #
    #
    # initialises the environment - can be called after you have logged in do all the
    # setting up of the different managed objects, rather than manually calling each
    # of the above methods
    def initEnv

      setHostSystem
      setConninfo
      setConfigManager
      setRootFolder
      setDatacenter
      setDatastore

    end # end initEnv

  end # end Vi module


  #
  #
  #
  # An ESXServer object describes an ESX server. On this object you can call
  # various methods - the majority of which are included from the Vi module
  class ESXServer

    include Vi # include all of the checks

    def initialize(ip)
      @ip = ip
      webservice = "https://"+@ip.to_s+"/sdk"
      @driver = VimPortType.new(webservice)

      # ignore SSL warnings
      @driver.options["protocol.http.ssl_config.verify_mode"] = nil

      # if a -w flag is set then dump the SOAP requests and responses to STDOUT
      @driver.wiredump_dev = STDOUT if ARGV[1].eql?("-w")

      # retrieve the service content as this will be needed by any methods
      svcref = ManagedObjectReference.new("ServiceInstance")
      svcref.xmlattr_type = "ServiceInstance"
      rservrtype = RetrieveServiceContentRequestType.new(svcref)
      @servicecontent = @driver.retrieveServiceContent(rservrtype)

      @hostSystem = nil # the host system used in the request for connection info
      @conninfo = nil # this needs setting but requires authentication first
      @configManager = nil # this can be set with the getConfigManager method
      @rootFolder = nil
      @datacenter = nil

    end # end initialize

  end # end ESXServer class

end # end VMware module
